@@ -1,21 +1,40 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 3113 $
-# Date: $Date: 2005-03-25 04:02:32 +0100 (Fri, 25 Mar 2005) $
+# $Id: test_enumerated_lists.py 9277 2022-11-26 23:15:13Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
 Tests for states.py.
 """
 
-from __init__ import DocutilsTestSupport
+from pathlib import Path
+import sys
+import unittest
+
+if __name__ == '__main__':
+    # prepend the "docutils root" to the Python library path
+    # so we import the local `docutils` package.
+    sys.path.insert(0, str(Path(__file__).resolve().parents[3]))
+
+from docutils.frontend import get_default_settings
+from docutils.parsers.rst import Parser
+from docutils.utils import new_document
+
+
+class ParserTestCase(unittest.TestCase):
+    def test_parser(self):
+        parser = Parser()
+        settings = get_default_settings(Parser)
+        settings.warning_stream = ''
+        for name, cases in totest.items():
+            for casenum, (case_input, case_expected) in enumerate(cases):
+                with self.subTest(id=f'totest[{name!r}][{casenum}]'):
+                    document = new_document('test data', settings.copy())
+                    parser.parse(case_input, document)
+                    output = document.pformat()
+                    self.assertEqual(output, case_expected)
 
-def suite():
-    s = DocutilsTestSupport.ParserTestSuite()
-    s.generateTests(totest)
-    return s
 
 totest = {}
 
@@ -655,8 +674,14 @@ A. Einstein was a great influence on
 B. Physicist, who was a colleague of
 C. Chemist.  They all worked in
 Princeton, NJ.
+
+Using a non-breaking space as a workaround:
+
+A.\u00a0Einstein was a great influence on
+B. Physicist, who was a colleague of
+C. Chemist.  They all worked in
+Princeton, NJ.
 """,
-# @@@ I think this is the correct result, but I'm not certain:
 """\
 <document source="test data">
     <enumerated_list enumtype="upperalpha" prefix="" suffix=".">
@@ -672,6 +697,13 @@ Princeton, NJ.
     <paragraph>
         C. Chemist.  They all worked in
         Princeton, NJ.
+    <paragraph>
+        Using a non-breaking space as a workaround:
+    <paragraph>
+        A.\xa0Einstein was a great influence on
+        B. Physicist, who was a colleague of
+        C. Chemist.  They all worked in
+        Princeton, NJ.
 """],
 ["""\
 1. Item one: line 1,
@@ -811,9 +843,84 @@ ii. Item two.
         #. Item two.
         3. Item three.
 """],
+["""\
+z.
+x
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        z.
+        x
+"""],
+["""\
+3-space indent, with a trailing space:
+
+1. \n\
+   foo
+
+3-space indent, no trailing space:
+
+1.
+   foo
+
+2-space indent:
+
+1.
+  foo
+
+1-space indent:
+
+1.
+ foo
+
+0-space indent, not a list item:
+
+1.
+foo
+
+No item content:
+
+1.
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        3-space indent, with a trailing space:
+    <enumerated_list enumtype="arabic" prefix="" suffix=".">
+        <list_item>
+            <paragraph>
+                foo
+    <paragraph>
+        3-space indent, no trailing space:
+    <enumerated_list enumtype="arabic" prefix="" suffix=".">
+        <list_item>
+            <paragraph>
+                foo
+    <paragraph>
+        2-space indent:
+    <enumerated_list enumtype="arabic" prefix="" suffix=".">
+        <list_item>
+            <paragraph>
+                foo
+    <paragraph>
+        1-space indent:
+    <enumerated_list enumtype="arabic" prefix="" suffix=".">
+        <list_item>
+            <paragraph>
+                foo
+    <paragraph>
+        0-space indent, not a list item:
+    <paragraph>
+        1.
+        foo
+    <paragraph>
+        No item content:
+    <enumerated_list enumtype="arabic" prefix="" suffix=".">
+        <list_item>
+"""],
 ]
 
 
 if __name__ == '__main__':
-    import unittest
-    unittest.main(defaultTest='suite')
+    unittest.main()
