@@ -1,5 +1,5 @@
 #! /usr/bin/env python3
-# $Id: test_latex2e.py 9369 2023-05-02 23:04:27Z milde $
+# $Id: test_latex2e.py 9384 2023-05-11 15:50:42Z milde $
 # Author: engelbert gruber <grubert@users.sourceforge.net>
 # Copyright: This module has been placed in the public domain.
 
@@ -22,8 +22,14 @@ from docutils.core import publish_string
 
 # DATA_ROOT is ./test/data from the docutils root
 DATA_ROOT = Path(__file__).resolve().parents[1] / 'data'
-spam = os.path.relpath(DATA_ROOT/'spam').replace('\\', '/')
+
 ham = os.path.relpath(DATA_ROOT/'ham.tex').replace('\\', '/')
+spam = os.path.relpath(DATA_ROOT/'spam').replace('\\', '/')
+# workaround for PyPy (cf. https://sourceforge.net/p/docutils/bugs/471/)
+if sys.implementation.name == "pypy" and sys.version_info < (3, 10):
+    spampath = f"PosixPath('{spam}.sty')"
+else:
+    spampath = f"'{spam}.sty'"
 
 
 class WriterPublishTestCase(unittest.TestCase):
@@ -53,7 +59,8 @@ class WriterPublishTestCase(unittest.TestCase):
         settings['use_latex_toc'] = False
         for name, cases in samples_docutils_toc.items():
             for casenum, (rst_input, expected) in enumerate(cases):
-                with self.subTest(id=f'samples_docutils_toc[{name!r}][{casenum}]'):
+                id = f'samples_docutils_toc[{name!r}][{casenum}]'
+                with self.subTest(id=id):
                     output = publish_string(source=rst_input,
                                             writer_name=self.writer_name,
                                             settings_overrides=settings)
@@ -129,7 +136,8 @@ class WriterPublishTestCase(unittest.TestCase):
         settings['stylesheet_path'] = f'{spam},{ham}'
         for name, cases in samples_stylesheet.items():
             for casenum, (rst_input, expected) in enumerate(cases):
-                with self.subTest(id=f'samples_link_stylesheet[{name!r}][{casenum}]'):
+                id = f'samples_link_stylesheet[{name!r}][{casenum}]'
+                with self.subTest(id=id):
                     output = publish_string(source=rst_input,
                                             writer_name=self.writer_name,
                                             settings_overrides=settings)
@@ -143,7 +151,8 @@ class WriterPublishTestCase(unittest.TestCase):
         settings['warning_stream'] = ''
         for name, cases in samples_stylesheet_embed.items():
             for casenum, (rst_input, expected) in enumerate(cases):
-                with self.subTest(id=f'samples_embed_stylesheet[{name!r}][{casenum}]'):
+                id = f'samples_embed_stylesheet[{name!r}][{casenum}]'
+                with self.subTest(id=id):
                     output = publish_string(source=rst_input,
                                             writer_name=self.writer_name,
                                             settings_overrides=settings)
@@ -1224,10 +1233,11 @@ two stylesheet links in the header
 samples_stylesheet_embed['two-styles'] = [
 ["""two stylesheets embedded in the header""",
 head_template.substitute(dict(parts,
-stylesheet=r"""% Cannot embed stylesheet:
-%  [Errno 2] No such file or directory: '""" + spam + r""".sty'
-% embedded stylesheet: """ + ham + r"""
-\newcommand{\ham}{wonderful ham}
+stylesheet=f"""\
+% Cannot embed stylesheet:
+%  [Errno 2] No such file or directory: {spampath}
+% embedded stylesheet: {ham}
+\\newcommand{{\\ham}}{{wonderful ham}}
 
 """)) + r"""
 two stylesheets embedded in the header
